* DCHOICE.PRG - array display/selector routine

* On entry, calling routine must set up the following for DCHOICE :
*       - Array containing information to display in window
*       - Number of entries in the array
*       - Starting rec in Array to show (default=1)
*       - Window coordinates to display (Top Row, Left Column, Max Botton Row)
*       - Array containing vaild "inkey()" values to cause exit
*       - Number of entries in exit value array
*       - Title and Footer line text (optional)
*       - Highlight (reversed) if desired (True/False)
*       - Shadow if desired (True/False)
*   NOTE: DCHOICE will dynamically set the right colum
*         based on the width of the first array entry.
*   ALSO: If array contains fewer entries than the depth
*         of display area as called, DCHOICE will "float"
*         the bottom of the box to exactly contain the array.
*
* On return, calling routine is supplied with :
*       - the pointer to the "inkey()" value of which caused DCHOICE to exit
*       - the current position of the active entry in the array

Parameters TopRow, LeftCol, BotRow,;
           Array, Ceiling,;
           Exit_Val, ExitTop,;
           Shadow, HiLite,;
           TitleLn, FooterLn,;
           StRec
  * First 3  parameters are box corners of boxed display area
  *          (Top Row, Left Column, Bottom Row-Max)
  * Fourth   parameter is array name of display records
  * Fifth    parameter is number of records in array when called
  *   "         "      is active record number at return(or 0 for Esc)
  * Sixth    parameter is array name of "inkey()" valuse which cause exit
  * Seventh  parameter is number of values in exit array when called
  *   "         "      is pointer to value which caused exit
  * Eighth   parameter is .T. for shadowed window; .F. for no shadow
  * Nineth   parameter is .T. for reversed window; .F. for normal
  * Tenth    parameter is title text for window (optional)
  * Eleventh parameter is footer text for window (optional)
  * Twelveth parameter is actice record in array to show

Private Current, Row, EndTop, ShowTop
  * Current : index pointer to array, returned as 6th parameter
  * Row     : screen row of current selection
  * EndTop  : top rec to show for full screen display at bottom of array
  * ShowTop : top rec of current display
Private Top_Row, Left_Col, Bot_Row, Rt_Col, RtCol, ShwBot, ShwRt
  * first four variables are the limits of the display area for the array
  * RtCol   : right side of the box
  * ShwBot  : bottom of display (includes Shadow)
  * ShwRt   : right side of display (includes Shadow)
Private BoxLen, BoxWide, BoxErr, ncolor, hcolor
  * BoxLen  : length of display window for array
  * BoxWide : width of display window for array
  * BoxErr  : logical value =.F. if box fits on screen
  * ncolor  : "Normal" color attributes
  * hcolor  : "Highlight" color attributes for shadow
Private Title, Footer
  * Title   : display title text
  * Footer  : display footer text
Private Hi, StartRec
  * Hi      : "Highlight" flag
  * StartRec: first record to be active

set cursor off
keypress = 0                                && set to invalid for abort

d_incolr = setcolor()
Hi = .T.
if pcount() < 9                             && Set window color
  Hi = .F.
else
  Hi = HiLite
endif
if Hi                                       && <=== CUSTOMIZE for color
  ncolor = colorv
  hcolor = colorn
else
  ncolor = colorn
  hcolor = colorv
endif
setcolor(ncolor)

RtCol   = LeftCol + len(Array[1]) + 1
Top_Row = TopRow + 1                        && Used for display area of array
Left_Col= LeftCol + 1                            && Inside box
Bot_Row = BotRow - 1
Rt_Col  = RtCol - 1
ShwRt   = RtCol                             && Set variable Right and Bottom
ShwBot  = BotRow

bot_offset = 1
if Shadow
  bot_offset = bot_offset + 1
  ShwRt  = ShwRt  + 1
endif

if pcount() > 9                             && Set Title and Footer
  Title = TitleLn
else
  Title = ''
endif
if pcount() > 10
  Footer = FooterLn
else
  Footer = ''
endif
if pcount() > 11                            && Set first active record
  if StRec > Ceiling
    StartRec = Ceiling
  elseif StRec < 1
    StartRec = 1
  else
    StartRec = StRec
  endif
else
  StartRec = 1
endif

if '' <> alltrim(Title)                     && Adjust box for Title
  Top_Row = Top_Row + 2
endif
if '' <> alltrim(Footer)                         && and Footer
  Bot_Row = Bot_Row - 2
  bot_offset = bot_offset + 2
endif

if (Bot_Row - Top_Row + 1) > Ceiling        && Adjust Bot_Row for small array
  Bot_Row = Top_Row + Ceiling - 1
  ShwBot  = Bot_Row + bot_offset
endif

if ShwBot <> Bot_Row + bot_offset
  Bot_Row = ShwBot - bot_offset
endif

BoxErr = .F.
do ChkBox                                   && check if box is on-screen
if BoxErr
  keypress = 0
  Ceiling  = 0
  setcolor(d_incolr)
  return
endif

@ Top_Row - 1, LeftCol to Bot_Row + 1, RtCol double  && Draw display box

if Shadow                                   && Check to draw shadow
  setcolor(hcolor)
  shadowbx(ShwRt,TopRow + 1,ShwBot,Left_Col)
  setcolor(ncolor)
endif

BoxWide = RtCol  - LeftCol  - 1              && Set Title and Footer and boxes
BoxLen  = Bot_Row - Top_Row + 1
if '' <> alltrim(Title)
  if len(Title) < BoxWide
    Title = Title + space(BoxWide - len(Title))
  endif
  @ TopRow,LeftCol to TopRow + 2, RtCol double
  setcolor(hcolor)
  @ TopRow + 1,  LeftCol + 1 say Title
  setcolor(ncolor)
  @ Top_Row - 1, LeftCol     say ''
  @ Top_Row - 1, RtCol       say ''
endif
if '' <> alltrim(Footer)
  if len(Footer) < BoxWide
    Footer = Footer + space(BoxWide - len(Footer))
  endif
  @ Bot_Row + 1, LeftCol to Bot_Row + 3, RtCol double
  setcolor(hcolor)
  @ Bot_Row + 2, LeftCol + 1 say Footer
  setcolor(ncolor)
  @ Bot_Row + 1, LeftCol     say ''
  @ Bot_Row + 1, RtCol       say ''
endif
if BoxWide > 7
  mor_offset = (BoxWide / 2) - 4
  mor_up = ' More  '
  mor_dn = ' More  '
  mor_no = ''
else
  mor_offset = BoxWide - 2
  mor_up = '  '
  mor_dn = '  '
  mor_no = ''
endif
if mor_offset < 1
  mor_offset = 1
endif

* Find end rec to show for full screen display at bottom of array
EndTop = Ceiling - BoxLen + 1

* Find the first rec to show for StartRec on screen
if StartRec < BoxLen
  ShowTop = 1
elseif StartRec > EndTop
  ShowTop = EndTop
else
  ShowTop = StartRec - int(BoxLen / 2)
endif

* Show as many recs as will fit in display area
if Ceiling > 0                              && if not an empty array
  Current = StartRec
  do DrawScrn with ShowTop
else
  keyboard chr(k_INS)                       && else expect to add new rec
endif

** Control Loop - intercept cursor keystrokes and process accordingly
do while .T.
  keypress = inkey(0)

  ExitIx_ = 1                               && Was the key pressed one of
  do while ExitIx_ < ExitTop + 1                 && the Exit Values provided?
    if keypress = Exit_Val[ExitIx_]
      Ceiling = Current                          && Yes-Save pointers and Exit
      ExitTop = ExitIx_
      setcolor(d_incolr)
      return
    endif
    ExitIx_  = ExitIx_ + 1
  enddo

  DO CASE                                        && NO-Check preset keys

  CASE keypress = k_RETURN                  && [Enter] key selected item
    Ceiling = Current
    ExitTop = ExitIx_
    exit

  CASE keypress = k_HELP               && [F1] key:'inkey(0)'not "Wait state"
    do help with 'DCHOICE',259,'KEYPRESS'

  CASE keypress = k_ESC                     && [Esc] key
    Ceiling = 0
    ExitTop = 0
    exit

  CASE keypress = k_CURS_UP                 && Up arrow pressed
    if Row = Top_Row
      if Current = 1                                  && attempt to cursor past
        keyboard chr(k_END)                           && top of list, go to end
      else
        @ Row,Left_Col say Array[Current]             && top of window, scroll
        Current = Current - 1
        scroll (Top_Row,Left_Col,Bot_Row,Rt_Col,-1)
        ShowTop = ShowTop - 1
        do RowHi with Array[Current]
      endif
    else                                              && else move cursor up
      @ Row,Left_Col say Array[Current]
      Current = Current - 1
      Row     = Row - 1
      do RowHi with Array[Current]
    endif

  CASE keypress = k_CURS_DN                 && Down arrow pressed
    if Row >= Bot_Row
      if Current = Ceiling                           && attempt to cursor past
        keyboard chr(k_HOME)                         && bottom of list, go top
      else
        @ Row,Left_Col say Array[Current]            && bottom of window, scroll
        Current = Current + 1
        scroll (Top_Row,Left_Col,Bot_Row,Rt_Col,1)
        ShowTop = ShowTop + 1
        do RowHi with Array[Current]
      endif
    else                                         && else move cursor down
      @ Row,Left_Col say Array[Current]
      Current = Current + 1
      Row     = Row + 1
      do RowHi with Array[Current]
    endif

  CASE keypress = k_HOME                    && Home Key pressed
    if Ceiling = BoxLen                          && array within window
      @ Row,Left_Col say Array[Current]
      Current = 1
      Row     = Top_Row
      do RowHi with Array[Current]
    else
      Current = 1
      ShowTop = 1
      do DrawScrn with ShowTop
    endif

  CASE keypress = k_END                     && End key pressed
    if Ceiling = BoxLen                          && array within window
      @ Row,Left_Col say Array[Current]
      Current = Ceiling
      Row     = Bot_Row
      do RowHi with Array[Current]
    else
      Current = Ceiling                          && redraw bottom of array
      ShowTop = EndTop
      do DrawScrn with ShowTop
    endif

  CASE keypress = k_PGUP                    && PgUp key pressed
    if Current = 1                               && at top
      keyboard chr(k_END)                           && go to bottom of list
    elseif Current - BoxLen <= 1                 && attempt to go beyond top
      keyboard chr(k_HOME)                          && go to top
    elseif ShowTop - (BoxLen) < 1            && not enough room for page
      keyboard chr(k_HOME)
    else
      Current = Current - BoxLen             && draw previous page
      ShowTop = ShowTop - BoxLen
      do DrawScrn with ShowTop
    endif

  CASE keypress = k_PGDN                    && PgDn key pressed
    if Current = Ceiling                         && at bottom
      keyboard chr(k_HOME)                          && go to top of list
    elseif Current + BoxLen >= Ceiling           && attempt to go beyond end
      keyboard chr(k_END)                           && go to end
    elseif ShowTop + (BoxLen * 2) > Ceiling      && not enough room for page
      keyboard chr(k_END)
    else
      Current = Current + BoxLen                 && draw next page
      ShowTop = ShowTop + BoxLen
      do DrawScrn with ShowTop
    endif

  ENDCASE

enddo
setcolor(d_incolr)
return

* =============>> PROCEDURES <<======================

procedure ChkBox                       && A Debug routine-Is box on-screen?
  if TopRow > BotRow
    do dsmsg with 'Top of Box is lower than Bottom. Fix it'
    BoxErr = .T.
  endif
  if TopRow > 23 .or. TopRow < 0
    xx = iif (TopRow > 23, TopRow - 23, abs(TopRow))
    do dsmsg with 'Top of Box is off the screen ' + alltrim(str(xx,2)) + ;
                  ' rows. Fix it'
    BoxErr = .T.
  endif
  if ShwBot > 24 .or. ShwBot < 1
    xx = iif (ShwRow > 24, ShwRow - 24, abs(TopRow))
    do dsmsg with 'Bottom of Box is off the screen ' + alltrim(str(xx,2)) + ;
                  ' rows. Fix it'
    BoxErr = .T.
  endif
  if LeftCol > 77 .or. LeftCol < 0
    xx = iif (LeftCol > 77, LeftCol - 77, abs(LeftCol))
    do dsmsg with 'Left side of Box is off the screen ' + alltrim(str(xx,2)) + ;
                  ' cols. Fix it'
    BoxErr = .T.
  endif
  if ShwRt > 79 .or. ShwRt < 2
    xx = iif (ShwRt > 79, ShwRt - 79, 2 - ShwRt)
    do dsmsg with 'Right side of Box is off the screen ' + alltrim(str(xx,2)) + ;
                  ' cols. Fix it'
    BoxErr = .T.
  endif
  if (Bot_Row - Top_Row < 0) .or. (Rt_Col - Left_Col < 2)
    do dsmsg with 'Box is too small to show anything. Fix it'
    BoxErr = .T.
  endif
return

procedure DrawScrn
  Row     = Top_Row
  ShowRec = ShowTop
  setcolor(ncolor)
  do while Row <= Bot_Row
    if Current = ShowRec                    && Highlight the current record
      do RowHi with Array[ShowRec]
    else
      @ Row,Left_Col say Array[ShowRec]
    endif
    Row     = Row + 1
    ShowRec = ShowRec + 1
  enddo
  Row = Top_Row + (Current - ShowTop)
return

procedure RowHi
parameter HiTxt
  setcolor(hcolor)
  @ Row,Left_Col say HiTxt
  setcolor(ncolor)
  if ShowTop > 1
    @ Top_Row - 1,LeftCol + mor_offset say mor_up
  else
    @ Top_Row - 1,LeftCol + mor_offset say mor_no
  endif
  if ShowTop = EndTop
    @ Bot_Row + 1,LeftCol + mor_offset say mor_no
  else
    @ Bot_Row + 1,LeftCol + mor_offset say mor_dn
  endif
return

function shadowbx
* draws shadow lines for box in active color
parameters top_col_,top_line_,bot_line_,bot_col_
  shadx = top_line_
  do while shadx < bot_line_
    @ shadx,top_col_ say ' '
    shadx = shadx + 1
  enddo
  @ bot_line_,bot_col_ say space(top_col_ - bot_col_ + 1)
return (.T.)

